@@ -1,12 +1,13 @@
 #  GTK+ based frontend to software-properties
 #
-#  Copyright (c) 2004-2018 Canonical Ltd.
+#  Copyright (c) 2004-2020 Canonical Ltd.
 #                2004-2005 Michiel Sikkes
 #
 #  Author: Michiel Sikkes <michiel@eyesopened.nl>
 #          Michael Vogt <mvo@debian.org>
 #          Sebastian Heinlein <glatzor@ubuntu.com>
 #          Andrea Azzarone <andrea.azzarone@canonical.com>
+#          Olivier Tilloy <olivier.tilloy@canonical.com>
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU General Public License as
@@ -29,12 +30,12 @@ import apt
 import apt_pkg
 import datetime
 import dbus
+import dbus.glib
+import dbus.mainloop.glib
 from gettext import gettext as _
 import gettext
 import os
 import subprocess
-from aptdaemon import client
-from aptdaemon.errors import NotAuthorizedError, TransactionFailed
 import logging
 import threading
 import sys
@@ -43,6 +44,8 @@ import time
 import gi
 gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
+gi.require_version("PackageKitGlib", "1.0")
+from gi.repository import PackageKitGlib as packagekit
 from gi.repository import GObject, Gdk, Gtk, Gio, GLib
 
 from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
@@ -65,6 +68,7 @@ from softwareproperties.gtk.utils import (
     is_current_distro_lts,
 )
 
+dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
 
 if GLib.pyglib_version < (3, 9, 1):
     GLib.threads_init()
@@ -88,6 +92,52 @@ RESPONSE_ADD = 2
     STORE_VISIBLE
 ) = list(range(5))
 
+# rows for the updates subscription combobox
+(
+    UPDATES_ALL,
+    UPDATES_RECOMMENDED,
+    UPDATES_SECURITY,
+    UPDATES_CUSTOM,
+) = range(4)
+
+# names of the pockets
+POCKET_SECURITY = "security"
+POCKET_UPDATES = "updates"
+POCKET_BACKPORTS = "backports"
+
+
+class UpdateSubscriptions(object):
+    security = False
+    updates = False
+    backports = False
+    other = False
+    inconsistent = False
+
+    def get_status(self):
+        if self.other:
+            return UPDATES_CUSTOM
+        if self.security and self.updates and self.backports:
+            return UPDATES_ALL
+        elif self.security and self.updates and not self.backports:
+            return UPDATES_RECOMMENDED
+        elif self.security and not self.updates and not self.backports:
+            return UPDATES_SECURITY
+        return UPDATES_CUSTOM
+
+
+def maybe_log_authentication_canceled_error(dbus_exception):
+    """Log an error if the DBus exception was raised because the user canceled
+    the authentication prompt shown when changing apt sources/configuration.
+
+    Return True if it is, so that calling code can do further processing,
+    False otherwise.
+    """
+    if dbus_exception._dbus_error_name == \
+      'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
+        logging.error("Authentication canceled, changes have not been saved")
+        return True
+    return False
+
 
 def error(parent_window, summary, msg):
     """ show a error dialog """
@@ -102,18 +152,6 @@ def error(parent_window, summary, msg):
     return False
 
 
-def get_dependencies(apt_cache, package_name, pattern=None):
-    """ Get the package dependencies, which can be filtered out by a pattern """
-    dependencies = []
-    for or_group in apt_cache[package_name].candidate.dependencies:
-        for dep in or_group:
-            if dep.rawtype in ["Depends", "PreDepends"]:
-                dependencies.append(dep.name)
-    if pattern:
-        dependencies = [ x for x in dependencies if x.find(pattern) != -1 ]
-    return dependencies
-
-
 class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
     def __init__(self, datadir=None, options=None, file=None, parent=None):
@@ -186,7 +224,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.handlers = {}
 
         self.apt_cache = {}
-        self.apt_client = client.AptClient()
+        self.pk_task = None
 
         # Put some life into the user interface:
         self.init_auto_update()
@@ -354,10 +392,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.vbox_dist_comps.add(checkbox)
             checkbox.show()
 
-        # Setup the checkbuttons for the child repos / updates
-        for checkbutton in self.vbox_updates.get_children():
-            self.vbox_updates.remove(checkbutton)
-
         status = get_ua_status()
         if not is_current_distro_lts():
                 esm_available = False
@@ -409,6 +443,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         )
         eol_expiry_text = _("Ended %s") % eol_date.strftime("%x")
 
+        # Setup the combo box for updates subscriptions
+        combobox = self.combobox_updates_subscription
+        self.handlers[combobox] = combobox.connect(
+            "changed", self.on_combobox_updates_subscription_changed)
+
         # setup the server chooser
         cell = Gtk.CellRendererText()
         self.combobox_server.pack_start(cell, True)
@@ -435,12 +474,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         """Fill the distro user interface with life"""
         self.block_handlers()
 
-        # Enable or disable the child source checkbuttons
-        for checkbox in self.vbox_updates.get_children():
-            (active, inconsistent) = self.get_comp_child_state(checkbox.template)
-            checkbox.set_active(active)
-            checkbox.set_inconsistent(inconsistent)
-
         # Enable or disable the component checkbuttons
         for checkbox in self.vbox_dist_comps.get_children():
             # check if the comp is enabled
@@ -448,13 +481,47 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             checkbox.set_inconsistent(inconsistent)
             checkbox.set_active(active)
 
+        # Refresh the combo box for updates subscriptions
+        subscriptions = UpdateSubscriptions()
+        for template in self.distro.source_template.children:
+            # Do not show source entries in there
+            if template.type == "deb-src":
+                continue
+            pocket = template.name.split('-')[-1]
+            (active, inconsistent) = self.get_comp_child_state(template)
+            subscriptions.inconsistent |= inconsistent
+            if active:
+                if pocket == POCKET_SECURITY:
+                    subscriptions.security = True
+                elif pocket == POCKET_UPDATES:
+                    subscriptions.updates = True
+                elif pocket == POCKET_BACKPORTS:
+                    subscriptions.backports = True
+                else:
+                    subscriptions.other = True
+
+        status = subscriptions.get_status()
+        combobox = self.combobox_updates_subscription
+        model = combobox.get_model()
+        if status == UPDATES_CUSTOM:
+            try:
+                model.get_iter_from_string(str(UPDATES_CUSTOM))
+            except ValueError:
+                model.append([_("Custom")])
+        else:
+            try:
+                model.remove(model.get_iter_from_string(str(UPDATES_CUSTOM)))
+            except ValueError:
+                pass
+        combobox.set_active(status)
+        combobox.set_sensitive(not subscriptions.inconsistent)
+
         # If no components are enabled there will be no need for updates
         # and source code
         if len(self.distro.enabled_comps) < 1:
-            self.vbox_updates.set_sensitive(False)
+            self.combobox_updates_subscription.set_sensitive(False)
             self.checkbutton_source_code.set_sensitive(False)
         else:
-            self.vbox_updates.set_sensitive(True)
             self.checkbutton_source_code.set_sensitive(True)
 
         # Check for source code sources
@@ -540,9 +607,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             if state != self.get_update_automation_level():
                 self.backend.SetUpdateAutomationLevel(state)
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
-
+            if maybe_log_authentication_canceled_error(e):
                 combo_handler = self.handlers[self.combobox_security_updates]
                 self.combobox_security_updates.handler_block(combo_handler)
                 self.set_security_update_level()
@@ -567,15 +632,52 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         try:
             self.backend.SetReleaseUpgradesPolicy(i)
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
-
+            if maybe_log_authentication_canceled_error(e):
                 combo_handler = self.handlers[self.combobox_release_upgrades]
                 self.combobox_release_upgrades.handler_block(combo_handler)
                 i = self.get_release_upgrades_policy()
                 self.combobox_release_upgrades.set_active(i)
                 self.combobox_release_upgrades.handler_unblock(combo_handler)
 
+    def on_combobox_updates_subscription_changed(self, combobox):
+        index = combobox.get_active()
+        if index != UPDATES_CUSTOM:
+            model = combobox.get_model()
+            try:
+                model.remove(model.get_iter_from_string(str(UPDATES_CUSTOM)))
+            except ValueError:
+                pass
+
+        def _toggle_child_source(pocket, enabled):
+            source = "{}-{}".format(self.distro.codename, pocket)
+            if enabled:
+                logging.info("enabling {}".format(source))
+                self.backend.EnableChildSource(source)
+            else:
+                logging.info("disabling {}".format(source))
+                self.backend.DisableChildSource(source)
+
+        try:
+            if index == UPDATES_ALL:
+                _toggle_child_source(POCKET_SECURITY, True)
+                _toggle_child_source(POCKET_UPDATES, True)
+                _toggle_child_source(POCKET_BACKPORTS, True)
+            elif index == UPDATES_RECOMMENDED:
+                _toggle_child_source(POCKET_SECURITY, True)
+                _toggle_child_source(POCKET_UPDATES, True)
+                _toggle_child_source(POCKET_BACKPORTS, False)
+            elif index == UPDATES_SECURITY:
+                _toggle_child_source(POCKET_SECURITY, True)
+                _toggle_child_source(POCKET_UPDATES, False)
+                _toggle_child_source(POCKET_BACKPORTS, False)
+            for template in self.distro.source_template.children:
+                if template.type == "deb-src":
+                    continue
+                pocket = template.name.split('-')[-1]
+                if pocket not in (POCKET_SECURITY, POCKET_UPDATES, POCKET_BACKPORTS):
+                    _toggle_child_source(pocket, False)
+        except dbus.DBusException as e:
+            maybe_log_authentication_canceled_error(e)
 
     def on_combobox_server_changed(self, combobox):
         """
@@ -603,8 +705,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                 self.active_server = combobox.get_active()
                 self.backend.ChangeMainDownloadServer(uri)
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
         # mvo: is this still needed?
         #else:
@@ -621,8 +722,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             else:
                 self.backend.DisableComponent(comp)
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_checkbutton_child_toggled(self, checkbutton, template):
         """
@@ -634,8 +734,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             else:
                 self.backend.EnableChildSource(template.name)
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_checkbutton_source_code_toggled(self, checkbutton):
         """ Disable or enable the source code for all sources """
@@ -645,8 +744,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             else:
                 self.backend.DisableSourceCodeSources()
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_checkbutton_popcon_toggled(self, widget):
         """ The user clicked on the popcon paritipcation button """
@@ -656,8 +754,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             try:
                 self.backend.SetPopconPariticipation(widget.get_active())
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
+                maybe_log_authentication_canceled_error(e)
 
     def open_file(self, file):
         """Show a confirmation for adding the channels of the specified file"""
@@ -675,8 +772,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                     for source in new_sources:
                         self.backend.AddSourceFromLine(str(source))
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_sources_drag_data_received(self, widget, context, x, y,
                                       selection, target_type, timestamp):
@@ -787,8 +883,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         try:
             self.backend.ToggleSourceUse(str(source_entry))
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def init_keys(self):
         """Setup the user interface parts needed for the key handling"""
@@ -849,8 +944,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         try:
             self.backend.Revert()
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
         if self.settings:
             self.settings.set_int("regular-auto-launch-interval", self.initial_auto_launch)
         self.show_auto_update_level()
@@ -934,9 +1028,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             try:
                 self.backend.SetUpdateInterval(value)
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
-
+                if maybe_log_authentication_canceled_error(e):
                     update_days = self.get_update_interval()
                     combo_handler = self.handlers[self.combobox_update_interval]
                     for key in self.combobox_interval_mapping:
@@ -955,8 +1047,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             try:
                 self.backend.AddSourceFromLine(line)
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
+                maybe_log_authentication_canceled_error(e)
 
     def on_edit_clicked(self, widget):
         """Show a dialog to edit an ISV source"""
@@ -972,8 +1063,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                 self.backend.ReplaceSourceEntry(str(old_source_entry),
                                                 str(dialog.new_source_entry))
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
+                maybe_log_authentication_canceled_error(e)
 
     # FIXME:outstanding from merge
     def on_isv_source_activated(self, treeview, path, column):
@@ -1013,8 +1103,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             try:
                 self.backend.RemoveSource(str(source_entry))
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
+                maybe_log_authentication_canceled_error(e)
 
     def add_key_clicked(self, widget):
         """Provide a file chooser that allows to add the gnupg of a trusted
@@ -1037,8 +1126,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                           _("The selected file may not be a GPG key file "
                             "or it might be corrupt."))
             except dbus.DBusException as e:
-                if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                    logging.error("Authentication canceled, changes have not been saved")
+                maybe_log_authentication_canceled_error(e)
 
     def remove_key_clicked(self, widget):
         """Remove a trusted software vendor key"""
@@ -1048,22 +1136,20 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             return
         key = model.get_value(a_iter, 0)
         try:
-            if not self.backend.RemoveKey(key[:8]):
+            if not self.backend.RemoveKey(key[:16]):
                 error(self.main,
                       _("Error removing the key"),
                       _("The key you selected could not be removed. "
                         "Please report this as a bug."))
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_restore_clicked(self, widget):
         """Restore the original keys"""
         try:
             self.backend.UpdateKeys()
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
     def on_delete_event(self, widget, args):
         """Close the window if requested"""
@@ -1086,10 +1172,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         try:
             self.backend.AddCdromSource()
         except dbus.DBusException as e:
-            if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
-                logging.error("Authentication canceled, changes have not been saved")
+            maybe_log_authentication_canceled_error(e)
 
-    def on_driver_changes_progress(self, transaction, progress):
+    def on_driver_changes_progress(self, progress, ptype, data=None):
         #print(progress)
         self.button_driver_revert.set_visible(False)
         self.button_driver_apply.set_visible(False)
@@ -1099,63 +1184,75 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.progress_bar.set_visible(True)
 
         self.label_driver_action.set_label(_("Applying changes..."))
-        self.progress_bar.set_fraction(progress / 100.0)
+        if ptype == packagekit.ProgressType.PERCENTAGE:
+            prog_value = progress.get_property('percentage')
+            self.progress_bar.set_fraction(prog_value / 100.0)
 
-    def on_driver_changes_finish(self, transaction, exit_state):
-        self.progress_bar.set_visible(False)
-        self.clear_changes()
-        self.apt_cache = apt.Cache()
-        self.set_driver_action_status()
-        self.update_label_and_icons_from_status()
-        self.button_driver_revert.set_visible(True)
-        self.button_driver_apply.set_visible(True)
-        self.button_driver_cancel.set_visible(False)
-        self.scrolled_window_drivers.set_sensitive(True)
-
-    def on_driver_changes_error(self, transaction, error_code, error_details):
-        self.on_driver_changes_revert()
-        self.set_driver_action_status()
-        self.update_label_and_icons_from_status()
-        self.button_driver_revert.set_visible(True)
-        self.button_driver_apply.set_visible(True)
-        self.button_driver_cancel.set_visible(False)
-        self.scrolled_window_drivers.set_sensitive(True)
-
-    def on_driver_changes_cancellable_changed(self, transaction, cancellable):
-        self.button_driver_cancel.set_sensitive(cancellable)
+    def on_driver_changes_finish(self, source, result, installs_pending):
+        results = None
+        try:
+            results = self.pk_task.generic_finish(result)
+        except Exception as e:
+            self.on_driver_changes_revert()
+            error(self.window_main, _("Error while applying changes"), str(e))
+        if not installs_pending:
+            self.progress_bar.set_visible(False)
+            self.clear_changes()
+            self.apt_cache = apt.Cache()
+            self.set_driver_action_status()
+            self.update_label_and_icons_from_status()
+            self.button_driver_revert.set_visible(True)
+            self.button_driver_apply.set_visible(True)
+            self.button_driver_cancel.set_visible(False)
+            self.scrolled_window_drivers.set_sensitive(True)
 
     def on_driver_changes_apply(self, button):
-
+        self.pk_task = packagekit.Task()
         installs = []
         removals = []
 
         for pkg in self.driver_changes:
             if pkg.is_installed:
-                removals.append(pkg.shortname)
+                removals.append(self.get_package_id(pkg.installed))
                 # The main NVIDIA package is only a metapackage.
                 # We need to collect its dependencies, so that
                 # we can uninstall the driver properly.
                 if 'nvidia' in pkg.shortname:
-                    for dep in get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
+                    for dep in self.get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
                         dep_pkg = self.apt_cache[dep]
                         if dep_pkg.is_installed:
-                            removals.append(dep_pkg.shortname)
+                            removals.append(self.get_package_id(dep_pkg.installed))
             else:
-                installs.append(pkg.shortname)
+                installs.append(self.get_package_id(pkg.candidate))
 
+        self.cancellable = Gio.Cancellable()
         try:
-            self.transaction = self.apt_client.commit_packages(install=installs, remove=removals,
-                                                               reinstall=[], purge=[], upgrade=[], downgrade=[])
-            self.transaction.connect("progress-changed", self.on_driver_changes_progress)
-            self.transaction.connect("cancellable-changed", self.on_driver_changes_cancellable_changed)
-            self.transaction.connect("finished", self.on_driver_changes_finish)
-            self.transaction.connect("error", self.on_driver_changes_error)
-            self.transaction.set_debconf_frontend("gnome")
-            self.transaction.run()
+            if removals:
+                installs_pending = False
+                if installs:
+                    installs_pending = True
+                self.pk_task.remove_packages_async(removals,
+                            False,  # allow deps
+                            True,  # autoremove
+                            self.cancellable,  # cancellable
+                            self.on_driver_changes_progress,
+                            (None, ),  # progress data
+                            self.on_driver_changes_finish,  # callback ready
+                            installs_pending  # callback data
+                 )
+            if installs:
+                self.pk_task.install_packages_async(installs,
+                        self.cancellable,  # cancellable
+                        self.on_driver_changes_progress,
+                        (None, ),  # progress data
+                        self.on_driver_changes_finish,  # GAsyncReadyCallback
+                        False  # ready data
+                 )
+
             self.button_driver_revert.set_sensitive(False)
             self.button_driver_apply.set_sensitive(False)
             self.scrolled_window_drivers.set_sensitive(False)
-        except (NotAuthorizedError, TransactionFailed) as e:
+        except Exception as e:
             print("Warning: install transaction not completed successfully: {}".format(e))
 
     def on_driver_changes_revert(self, button_revert=None):
@@ -1175,7 +1272,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.button_driver_apply.set_sensitive(False)
 
     def on_driver_changes_cancel(self, button_cancel):
-        self.transaction.cancel()
+        self.cancellable.cancel()
         self.clear_changes()
 
     def on_driver_restart_clicked(self, button_restart):
